

import pytest
from pydelaunator import Mesh, Edge, Face, Vertex


@pytest.fixture
def mesh():
    return Mesh(100, 100)


def test_object_corvering(mesh):
    face = mesh.object_covering(10, 10)
    assert isinstance(face, Face)

    edge = mesh.object_covering(50, 50)
    assert isinstance(edge, Edge)

    vrtx = mesh.object_covering(0, 0)
    assert isinstance(vrtx, Vertex)


def test_mesh_dim(mesh):
    assert mesh.corrected_position(0, 100) == (1, 99)
    assert mesh.corrected_position(50, 50) == (50, 50)

def test_mesh_add(mesh):
    mesh.add('one', 10, 10)
    with pytest.raises(ValueError):
        mesh.add('another at the same place', 10, 10)


def test_mesh(mesh):
    assert len(mesh.edges) == 12
    assert len(mesh.corners) == 4
    assert len(mesh.vertices) == 4
    assert mesh.vertices == mesh.corners
    added = mesh.add('object', 50, 50)
    assert mesh.vertices != mesh.corners
    assert len(mesh.vertices) == 5
    assert added in mesh.vertices
    assert added not in mesh.corners
    added = mesh.remove(added)
    assert len(mesh.edges) == 12
    assert len(mesh.corners) == 4
    assert len(mesh.vertices) == 4


def test_remove():
    # reproduction of a weird case encountered while playing
    mesh = Mesh(600, 600)
    points = (75, 381), (133, 192), (320, 107), (503, 488), (528, 209), (465, 325)
    for point in points:
        mesh.add(str(point), *point)
    todel = (251, 574)
    vertex = mesh.add(str(todel), *todel)
    mesh.move(vertex, 271, 540)  # just change the position
    mesh.move(vertex, 271, 539)  # delete and replace it


def test_neighbors(mesh):
    for vertex in mesh.vertices:
        nb_nei = len(tuple(vertex.direct_neighbors))
        assert nb_nei == 3
        assert nb_nei == len(set(vertex.direct_neighbors))


def test_neighbors_more(mesh):
    added_vertex = mesh.add('object', 50, 50)
    assert len(tuple(added_vertex.direct_neighbors)) == 4
    for vertex in mesh.vertices:
        nb_nei = len(tuple(vertex.direct_neighbors))
        assert nb_nei in {3, 4}
        assert nb_nei == len(set(vertex.direct_neighbors))






import argparse


def parse_cli() -> dict:
    parser = argparse.ArgumentParser()
    parser.add_argument('--universe-size', '-u', type=tuple, default=(600, 600),
                        help="size of universe/window.")
    parser.add_argument('--padding', '-p', type=int, default=50,
                        help="space between triangulation and window border.")
    parser.add_argument('--fps', '-f', type=int, default=10,
                        help="frame per second.")
    parser.add_argument('--speed', '-s', type=float, default=0.01,
                        help="Time between ticks.")
    parser.add_argument('--mouse_precision', '-m', type=int, default=400,
                        help="Click detection precision.")
    return dict(parser.parse_args()._get_kwargs())


if __name__ == "__main__":
    from pydelaunator import gui
    gui.run(**parse_cli())





from itertools import zip_longest, islice
from collections import deque
from pydelaunator.setup_logging import logger


def grouped(iterable:iter, by:int, fillvalue=None) -> iter:
    """Collect data into fixed-length chunks or blocks

    >>> tuple(grouped('ABCDEFG', 3, 'x'))
    (('A', 'B', 'C'), ('D', 'E', 'F'), ('G', 'x', 'x'))

    """
    args = [iter(iterable)] * by
    return zip_longest(*args, fillvalue=fillvalue)


def sliding(iterable:iter, size:int) -> iter:
    """Sliding window of given size of given iterable.

    >>> tuple(map(''.join, sliding('ABCD', 2)))
    ('AB', 'BC', 'CD')

    """
    iterable = iter(iterable)
    acc = deque(islice(iterable, 0, size), maxlen=size)
    yield tuple(acc)
    for elem in iterable:
        acc.append(elem)
        yield tuple(acc)


def draw_digraph(graph:iter, layout:dict=None):
    """Print given iterable of 2-uplet understood as edges"""
    import networkx as nx
    import matplotlib
    matplotlib.use("TkAgg")
    import matplotlib.pyplot as plt
    from matplotlib.lines import Line2D

    G = nx.DiGraph()
    G.add_edges_from(graph)
    graph_pos = layout or nx.shell_layout(G)
    nx.draw_networkx_nodes(G, graph_pos, node_size=1000, node_color='blue', alpha=0.3)
    nx.draw_networkx_edges(G, graph_pos, width=2, alpha=0.3, edge_color='green')
    nx.draw_networkx_labels(G, graph_pos, font_size=12, font_family='sans-serif')
    # matplotlib.pyplot.xkcd()
    plt.show()



"""Functions helping for navigation in the Mesh.

"""

import itertools
from math import inf
from operator import itemgetter
from collections import deque
from pydelaunator import Vertex
from pydelaunator.geometry import distance_between_points


def neighbors(vertex:Vertex, max_dist:float=inf, min_dist:float=0.) -> iter:
    assert vertex
    assert min_dist <= max_dist
    reference = vertex
    cur_edge = vertex.edge
    assert cur_edge
    found_neighbors = {vertex}  # set of found neighbors
    queue = deque([vertex])

    def found(v:Vertex) -> (Vertex) or ():
        if v not in found_neighbors:
            found_neighbors.add(v)
            dist = distance_between_points(*reference, *v)
            if min_dist <= dist <= max_dist:
                queue.append(v)
                yield v

    while queue:
        vertex = queue.popleft()
        for nei in vertex.direct_neighbors:
            yield from found(nei)


def nearests(vertex) -> iter:
    distance = lambda v: distance_between_points(*vertex, *v)
    distance_to_one_neighbor = distance(vertex.edge.target_vertex)
    all_neighbors = neighbors(vertex, max_dist=distance_to_one_neighbor)
    candidates = {neighbor: distance(neighbor) for neighbor in all_neighbors}
    # now extract the best of them (minimal distance)
    best_score = min(candidates.values())
    yield from (nei for nei, score in candidates.items() if score == best_score)





import math
import pytest
import itertools
from pydelaunator.geometry import *


def test_distance_between_points():
    assert square_distance_between_points(0, 0, 0, 0) == 0
    assert square_distance_between_points(0, 0, 1, 1) == 2

def test_square_distance_between_segment_and_point():
    d = math.sqrt(square_distance_between_segment_and_point(0,0, 1,1, 1,2))
    assert abs(d - 1.) < EPSILON

def test_point_collide_segment():
    assert point_collide_segment(0, 50, 0, 0, 0, 100)
    assert point_collide_segment(50, 50, 100, 0, 0, 100)

def test_point_in_clockwise_order():
    points = (0, 0), (0, 50), (50, 50), (50, 0)
    assert signed_polygon_area(points) == -signed_polygon_area(reversed(points))
    assert     point_in_clockwise_order(points)
    assert not point_in_counter_clockwise_order(points)
    assert not point_in_clockwise_order(reversed(points))
    assert     point_in_counter_clockwise_order(reversed(points))

def test_point_in_clockwise_order_2():
    points = (0, 0), (50, 50), (100, 0)
    assert signed_polygon_area(points) == -signed_polygon_area(reversed(points))
    assert     point_in_clockwise_order(points)
    assert not point_in_counter_clockwise_order(points)
    assert not point_in_clockwise_order(reversed(points))
    assert     point_in_counter_clockwise_order(reversed(points))

def test_point_in_counter_clockwise_order():
    points = (0, 0), (100, 0), (50, 50)
    assert signed_polygon_area(points) == -signed_polygon_area(reversed(points))
    assert not point_in_clockwise_order(points)
    assert     point_in_counter_clockwise_order(points)
    assert     point_in_clockwise_order(tuple(reversed(points)))
    assert not point_in_counter_clockwise_order(tuple(reversed(points)))


def test_signed_polygon_area():
    points = (5,0), (6,4), (4,5), (1,5), (1,0)
    assert signed_polygon_area(points) == -22
    assert point_in_counter_clockwise_order(points)
    assert signed_polygon_area(reversed(points)) == 22
    assert point_in_clockwise_order(reversed(points))


def test_signed_polygon_area_2():
    points = (3, 4), (5, 11), (12, 8), (9, 5), (5, 6), (3, 4)
    assert signed_polygon_area(points) == 30
    assert point_in_clockwise_order(points)
    assert signed_polygon_area(reversed(points)) == -30
    assert not point_in_clockwise_order(reversed(points))


def test_point_in_circumcircle_of():
    # aligned points
    assert not point_in_circumcircle_of((1,2), (0,2), (3,2), (1,-100))
    assert not point_in_circumcircle_of((0,2), (1,2), (3,2), (2,4))
    assert not point_in_circumcircle_of((2,1), (2,0), (2,3), (2,4))

    # non aligned points
    assert not point_in_circumcircle_of((1,0), (1,2), (0,1), (3-EPSILON,1))
     # 3 firsts are in counter-clockwise
    assert     point_in_circumcircle_of((1,0), (1,2), (0,1), (2-0.1,1))
     # 3 firsts are in counter-clockwise
    assert     point_in_circumcircle_of((1,0), (1,2), (0,1), (1-EPSILON,1))

def test_point_in_circumcircle_of_more():
    assert     point_in_circumcircle_of((0, 100), (100, 100), (100, 0), (70, 10))
    assert     point_in_circumcircle_of((0, 10), (4, 4), (10, 0), (10, 10))
    assert     point_in_circumcircle_of((0, 100), (40, 40), (100, 0), (100, 100))
    assert     point_in_circumcircle_of((0, 100), (70, 10), (100, 0), (100, 100))
    assert not point_in_circumcircle_of((100, 100), (100, 0), (70, 10), (0, 100))
    assert not point_in_circumcircle_of((40, 60), (40, 40), (80, 10), (100, 100))

def test_point_exactly_on_circumcircle():
    points = (0, 100), (0, 0), (100, 0), (100, 100)
    for combi in itertools.combinations(points, r=4):
        assert not point_in_circumcircle_of(*points)


def test_segment_cross_segment():
    points = (1, 3), (3, 3), (3, 1), (3, 3)
    assert segment_collides_line(*points) == 2
    assert     segment_collides_segment(*points)
    assert not segment_crosses_segment(*points)

def test_segment_cross_segment_2():
    points = (0, 1), (1, 0), (0, 0), (1, 1)
    assert segment_collides_line(*points) == 1
    assert segment_collides_segment(*points)
    assert segment_crosses_segment(*points)

def test_segment_cross_segment_3():
    points = (0, 1), (0, 0), (1, 0), (1, 1)
    assert segment_collides_line(*points) == 0
    assert not segment_collides_segment(*points)
    assert not segment_crosses_segment(*points)

def test_segment_cross_segment_4():
    points = (0, 1), (1, 1), (0, 2), (0, 0)
    assert segment_collides_line(*points) == 2
    assert     segment_collides_segment(*points)
    assert not segment_crosses_segment(*points)

def test_segment_cross_segment_5():
    points = (262, 542), (263, 541), (503, 488), (0, 600)
    assert segment_collides_line(*points) == 1
    assert     segment_collides_segment(*points)
    assert     segment_crosses_segment(*points)





from pydelaunator import geometry


class Face:
    """Triangle in Mesh space, surrounded by there Edge and three Vertex."""
    __slots__ = ['_edge']

    def __init__(self, edge=None):
        self._edge = edge

    @property
    def edge(self):
        return self._edge
    @edge.setter
    def edge(self, edge):
        edge._left_face = self
        self._edge = edge

    @property
    def surrounding_edges(self) -> tuple:
        """Return the 3 edges that surround self"""
        ret = (
            self.edge,
            self.edge.next_left_edge,
            self.edge.next_left_edge.next_left_edge,
        )
        assert all(edge.left_face is self for edge in ret)
        return ret

    @property
    def surrounding_vertices(self) -> tuple:
        """Return the 3 vertices that surround self"""
        return tuple(edge.target_vertex for edge in self.surrounding_edges)


    def circumcircle_contain_position(self, position:tuple) -> bool:
        """True if given position is in the circumcircle of the triangle
        formed by the 3 vertices of this face.

        """
        return geometry.point_in_circumcircle_of(*self.surrounding_vertices, position)

    def __str__(self):
        return "FACE({}->{}->{})".format(self._edge.origin_vertex, self._edge.next_left_edge.origin_vertex, self._edge.next_left_edge.next_left_edge.origin_vertex)





from pydelaunator import navigation
from pydelaunator import Mesh, Vertex
from pydelaunator.commons import logger


class Placer:
    """High level interface for the Mesh class.

    End-user can be totally abstracted from the Mesh and Vertex concepts.

    """

    def __init__(self, width:int, height:int):
        self.mesh = Mesh(width, height)
        self._objects = {}  # client object: vertex in mesh

    def add(self, obj:object, x:int, y:int) -> object:
        assert obj not in self._objects
        if obj in self._objects:
            i, j = self._objects[obj]
            logger.warning("Placer can't add object '{}' because it is already "
                           "placed (at ({};{}). It will be moved instead."
                           "".format(obj, i, j))
            self.move(obj, x - i, y - j)
        else:  # object is new
            added = self.mesh.add(obj, x, y)
            self._objects[obj] = added
            return obj

    def move(self, obj:object, dx:int, dy:int):
        self.mesh.move(self._objects[obj], dx, dy)

    def nearests(self, obj:object) -> iter:
        """Yield the nearests objects."""
        yield from (vertex.payload for vertex in navigation.nearests(self._objects[obj]))


if __name__ == "__main__":
    pass





import random
import pytest
from pydelaunator import Placer


class MockData:
    def __init__(self):
        self.data = random.randint(1, 1000)


@pytest.fixture
def placer():
    return Placer(1000, 1000)

@pytest.fixture
def social_network() -> dict:
    return {
        'Michel': (100, 100),
        'Gislaine': (500, 350),
        'Gérard': (200, 300),
        'Micheline': (400, 150),
    }

@pytest.fixture
def mockdata():
    return MockData(), MockData(), MockData(), MockData()


def test_basics(placer, social_network):
    for people, pos in social_network.items():
        placer.add(people, *pos)

def test_basic_navigation(placer, mockdata):
    one, two, tee, foo = mockdata
    assert isinstance(placer.add(one, 100, 100), MockData)
    assert isinstance(placer.add(two, 200, 100), MockData)
    assert isinstance(placer.add(tee, 200, 200), MockData)
    assert isinstance(placer.add(foo, 100, 200), MockData)
    placer.move(foo, 50, 0)
    nearests = tuple(placer.nearests(foo))
    assert len(nearests) == 1
    assert nearests[0] is tee



from pydelaunator import geometry


class Edge:
    __slots__ = ['_next_left_edge', '_origin_vertex', '_target_vertex',
                 '_left_face', '_opposite_edge', '_constrained']

    def __init__(self, origin_vertex, target_vertex, left_face=None,
                 next_left_edge=None, opposite_edge=None, constrained:bool=False):
        self._next_left_edge = next_left_edge
        self._origin_vertex = origin_vertex
        self._target_vertex = target_vertex
        self._left_face = left_face
        self._opposite_edge = opposite_edge
        self._constrained = bool(constrained)


    def nullify(self):
        """Remove all references to other objects. This is useful to ensure
        an error if this object is used after this call.
        """
        self._next_left_edge = None
        self._origin_vertex = None
        self._target_vertex = None
        self._left_face = None
        self._opposite_edge = None


    def squareDistanceTo(self, px, py) -> float:
        """Return square of distance between the edge and given coordinates."""
        return geometry.square_distance_between_segment_and_point(
            *self._origin_vertex.pos,
            *self._target_vertex.pos,
            px, py
        )

    def dot_product(self, pos) -> float:
        """Return dot product of given edge and point defined by given coordinates."""
        tx, ty = self._target_vertex.pos
        ox, oy = self._origin_vertex.pos
        px, py = pos
        return ((ty - oy) * (px - ox) + (ox - tx) * (py - oy))


    def coordOnTheRight(self, c) -> bool:
        """True if given coordinates are on the right of this Edge."""
        return self.dot_product(c) <= 0.
    def coordOnTheStrictRight(self, c) -> bool:
        """True if given coordinates are on the right of this Edge, but not on the Edge itself."""
        return self.dot_product(c) < 0.
    def coordOnTheLeft(self, c) -> bool:
        """True if given coordinates are on the left of this Edge."""
        return self.dot_product(c) >= 0.
    def coordOnTheStrictLeft(self, c) -> bool:
        """True if given coordinates are on the left of this Edge, but not on the Edge itself."""
        return self.dot_product(c) > 0.

    @property
    def origin_vertex(self):
        return self._origin_vertex
    @property
    def target_vertex(self):
        return self._target_vertex

    @property
    def next_left_edge(self):
        return self._next_left_edge
    @next_left_edge.setter
    def next_left_edge(self, next_left_edge):
        self._next_left_edge = next_left_edge
    @property
    def prev_left_edge(self):
        raise self._next_left_edge._next_left_edge

    @property
    def next_right_edge(self):
        return self.opposite_edge._next_left_edge._next_left_edge.opposite_edge
    @property
    def prev_right_edge(self):
        return self.opposite_edge._next_left_edge.opposite_edge

    @property
    def rot_right_edge(self):
        return self._opposite_edge._next_left_edge
    @property
    def rot_left_edge(self):
        return self._next_left_edge._next_left_edge.opposite_edge

    @property
    def left_face(self):
        return self._left_face
    @left_face.setter
    def left_face(self, left_face):
        self._left_face = left_face
    @property
    def right_face(self):
        return self.opposite_edge._left_face

    @property
    def opposite_edge(self):
        return self._opposite_edge
    @opposite_edge.setter
    def opposite_edge(self, opposite_edge):
        opposite_edge._opposite_edge = self
        self._opposite_edge = opposite_edge

    @property
    def constrained(self) -> bool:
        return self._constrained

    def __str__(self):
        return "EDGE{}({}->{})".format(('!' if self.constrained else ''),
                                       self.origin_vertex, self.target_vertex)



"""Setup the readings logger."""

import logging

PACKAGE_NAME = 'pydelaunator'


# Basic logging
logger = logging.getLogger(PACKAGE_NAME)
logger.setLevel(logging.DEBUG)


# Write in term
handler = logging.StreamHandler()
handler.setLevel(logging.WARNING)

formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
handler.setFormatter(formatter)

logger.addHandler(handler)


# Write in file
handler = logging.FileHandler('logs/{}.logs'.format(PACKAGE_NAME))
handler.setLevel(logging.DEBUG)

formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
handler.setFormatter(formatter)

logger.addHandler(handler)



"""Graphical representation of a delaunator triangulation.

Commands:
    ESC: quit
    ESPACE or ENTER: add a point at mouse position
    DELETE: remove point at mouse position
    drag&drop: point to move it
    a: toggle auto-adding points
    t: toggle random moving
    s: take a snapshot of all points positions

"""


import sys
import math
import time
import itertools
import functools
from random          import randint, choice

import pyglet
from pyglet.window   import key
from pyglet.window   import mouse

from pydelaunator    import Mesh




def run(universe_size:tuple=(600, 600), padding:int=50, fps:int=10,
        speed:float=0.01, mouse_precision:int=400):
    INTERFACE_TIME_SPEED    = float(speed)
    SIMULATION_FPS          = int(fps)
    MOUSE_PRECISION         = int(mouse_precision)
    UNIVERSE_SIZE           = tuple(map(int, universe_size))
    VIDEO_PADDING           = int(padding or 50)

    VIDEO_MODE_X            = UNIVERSE_SIZE[0] + VIDEO_PADDING*2
    VIDEO_MODE_Y            = UNIVERSE_SIZE[1] + VIDEO_PADDING*2
    PROGRAM_NAME            = "Delaunator 0.1"
    """Quick and dirty encapsulation of all the gui codebase"""
    # states
    auto_add = False
    auto_mov = False
    dragged_point = None
    mouse_position = None
    counter = 0
    dt = None

    # pyglet
    window = pyglet.window.Window(
        width=VIDEO_MODE_X, height=VIDEO_MODE_Y,
        caption=PROGRAM_NAME
    )

    # flags
    PRINT_EDGES = True
    class Point:
        """Objects put in the triangulation"""
        def __init__(self, color=None):
            self.color = color or tuple([100 + randint(0, 155) for _ in range(3)] + [255])


    def screen_position_to_dt_position(pos:tuple) -> tuple:
        x, y = pos
        return x - VIDEO_PADDING, y - VIDEO_PADDING
    def dt_position_to_screen_position(pos:tuple) -> tuple:
        x, y = pos
        return x + VIDEO_PADDING, y + VIDEO_PADDING


    def start_gui(dt_object):
        """dt must be a delaunator object"""
        # first printings
        nonlocal dt
        dt = dt_object
        print("WANDA JACKSON")
        print(__doc__)

        # just for begin…
        # for i in range(4):
            # _addPointToDT()
        # UNIT TEST 0
        # _addPointToDT(300, 300)
        # UNIT TEST 1
        # _addPointToDT(200, 100)
        # _addPointToDT(100,   0)
        # _addPointToDT(  0, 100)
        # _addPointToDT(100, 200)
        # _addPointToDT(221, 249)
        # _addPointToDT(258, 305)
        # UNIT TEST 2: test the face stacking when simplifying delaunay
        #   triangulation in suppression point algorithm.
        # _addPointToDT(121, 5)
        # _addPointToDT(161, 61)
        # _addPointToDT(211, 135)
        # _addPointToDT(282, 176)
        # _addPointToDT(357, 54) # its d
        # _addPointToDT(508, 253)
        # _addPointToDT(533, 52)
        # UNIT TEST 3
        # _addPointToDT(91, 372)
        # _addPointToDT(122, 309)
        # _addPointToDT(118, 204)
        # _addPointToDT(250, 575)
        # _addPointToDT(312, 107)
        # _addPointToDT(387, 204)
        # _addPointToDT(506, 480)
        # _addPointToDT(493, 216)
        # _addPointToDT(308, 373)
        # UNIT TEST 4
        _addPointToDT(75, 381)
        _addPointToDT(133, 192)
        _addPointToDT(249, 574)
        _addPointToDT(320, 107)
        _addPointToDT(503, 488)
        _addPointToDT(528, 209)
        _addPointToDT(465, 325)

        # run pyglet printings
        pyglet.app.run()
        pyglet.app.exit()
        print("\nWANDA JACKSOFF")


    @window.event
    def on_key_press(symbol, modifiers):
        nonlocal auto_add, mouse_position
        if symbol == key.RETURN or symbol == key.SPACE: # add point
            if mouse_position is not None:
                _addPointToDT(*mouse_position)
        elif symbol in (key.Q, key.ESCAPE):
            window.close()
        elif symbol in (key.A, key.B):
            if auto_add:
                pyglet.clock.unschedule(schedulable_addPointToDT)
            else:
                pyglet.clock.schedule_interval(
                    schedulable_addPointToDT,  # dt argument is useless
                    interval=INTERFACE_TIME_SPEED
                )
            auto_add = not auto_add
        elif symbol == key.P:
            dt.print()
        elif symbol in (key.M, key.T):
            _moveAllPoints()
        elif symbol in (key.S,):
            _snapshot()
        elif symbol == key.DELETE:  # del point
            if mouse_position is not None:
                _delPoint(_getPointAt(*mouse_position))


    @window.event
    def on_mouse_motion(x, y, dx, dy):
        nonlocal mouse_position
        mouse_position = screen_position_to_dt_position((x, y))

    @window.event
    def on_mouse_press(x, y, buttons, modifiers):
        nonlocal dt, mouse_position, dragged_point
        mouse_position = screen_position_to_dt_position((x, y))
        if buttons == pyglet.window.mouse.LEFT:
            dragged_point = _getPointAt(*mouse_position)

    @window.event
    def on_mouse_release(x, y, buttons, modifiers):
        nonlocal mouse_position, dragged_point
        mouse_position = screen_position_to_dt_position((x, y))
        if buttons == pyglet.window.mouse.LEFT:
            dragged_point = None
        if dragged_point is not None:
            movePoint(dx, dy, dragged_point)

    @window.event
    def on_mouse_drag(x, y, dx, dy, buttons, modifiers):
        nonlocal mouse_position, dragged_point
        if dragged_point is not None:
            _movePoint(dx, dy, dragged_point)

    @window.event
    def on_draw():
        nonlocal dt, mouse_position, dragged_point
        window.clear()
        _draw_edges()



    ################# DRAW METHODS #################
    def _draw_edges():
        """Print current state of engine"""
        nonlocal dt, dragged_point
        def print_edge(coords, color, width=1):
            pyglet.graphics.draw(2, pyglet.gl.GL_LINES,
                                     ('v2f', coords),
                                     ('c4B', color),
                                ) #TODO: use width

        if not PRINT_EDGES:
            return

        color_black = (0, 0, 0, 255)
        color_red   = (255, 0, 0, 255)
        diameter = 20
        for edge in dt.edges:
            if edge in dt.outside_objects:  continue
            source, target = edge.origin_vertex, edge.target_vertex
            p1 = dt_position_to_screen_position(source.position)
            p2 = dt_position_to_screen_position(target.position)
            if edge.constrained:
                color = color_red + color_red
            else:
                source_color = color_red if source is dragged_point else getattr(source.payload, 'color', color_black)
                target_color = color_red if target is dragged_point else getattr(target.payload, 'color', color_black)
                color = source_color + target_color
            print_edge((*p1, *p2), color)


    ################# POINT FUNCTIONS #################
    def schedulable_addPointToDT(dt, *args, **kwargs):
        return _addPointToDT(*args, **kwargs)

    def _addPointToDT(x=None, y=None):
        """Add a random point in DT"""
        nonlocal dt
        assert x is None or isinstance(x, int)
        assert y is None or isinstance(y, int)
        coords = x or randint(1, dt.width-1), y or randint(1, dt.height-1)
        # print(x, y)
        # print(coords)
        # print("Add point at ({};{})".format(*coords))
        dt.add(Point(), *coords)
        print('dt.add(Point(), {}, {})'.format(*coords))


    def _delPoint(point):
        """Delete given Point"""
        nonlocal dt, dragged_point
        if point is not None:
            if dragged_point == point:
                dragged_point = None
            try:
                dt.remove(point)
            except ValueError as err:
                print(err)


    def _getPointAt(x, y):
        """Return point that is at given coordinates (about MOUSE_PRECISION)"""
        nonlocal dt
        try:
            return next(iter(dt.vertices_at((x, y), MOUSE_PRECISION)))
        except StopIteration:
            return None


    def _movePoint(x, y, p):
        """Add given values to (x;y) of given point"""
        nonlocal dt
        if p is not None:
            try:
                dt.move(p, x, y)
                print('dt.move({}, {}, {})'.format(p, x, y))
            except AssertionError as err:
                import traceback
                traceback.print_tb(err.__traceback__)
                print(err)
                print(dragged_point)
                input('<get report>')
                print(dt)
                dt.term_print()
                dt.print()



    def _moveAllPoints():
        """Move all vertices by a small move"""
        nonlocal dt, dragged_point
        for v in dt.vertices:
            if v.payload:
                eps = 1
                mx = choice([-eps,eps])
                my = choice([-eps,eps])
                if v is not dragged_point:
                    dt.move(v, *(mx, my))


    def _snapshot():
        nonlocal dt
        print('\nsnapshot:')
        for c in (o.coordinates() for o in dt.trianguledObjects()):
            print('\t_addPointToDT' + str((c.x, c.y)))

    dt = Mesh(*UNIVERSE_SIZE)
    start_gui(dt)



import os
from setuptools   import setup, find_packages
from pip.req      import parse_requirements
from pip.download import PipSession


# access to the file at the package top level (like README)
def path_to(filename):
    return os.path.join(os.path.dirname(__file__), filename)

# parse_requirements() returns generator of pip.req.InstallRequirement objects
install_reqs = parse_requirements(path_to('requirements.txt'),
                                  session=PipSession())
reqs = [str(ir.req) for ir in install_reqs]


setup(
    name='pydelaunator',
    version='0.0.4',
    packages=find_packages(),
    include_package_data = True,  # read the MANIFEST.in file
    install_requires=reqs,

    author="lucas bourneuf",
    author_email="lucas.bourneuf@openmailbox.org",
    description="quad-edge data structure",
    long_description=open(path_to('README.mkd')).read(),
    keywords="graph data structure",
    url="https://github.com/Aluriak/pydelaunator",

    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
